@@ -1,6 +1,12 @@
 'use strict';
 
 const https = require('https');
+let http2;
+try {
+  http2 = require('http2'); // eslint-disable-line global-require
+} catch (_) {
+  // eslint-disable-line no-empty
+}
 const fs = require('fs');
 const path = require('path');
 const should = require('should');
@@ -9,6 +15,7 @@ const bodyParser = require('body-parser');
 const cookieParser = require('cookie-parser');
 const nock = require('nock');
 const request = require('../index.js');
+const throwError = require('./throwError');
 
 process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
 
@@ -20,14 +27,14 @@ function shouldIncludeStackWithThisFile(err) {
 describe('request(url)', function () {
   it('should be supported', function (done) {
     const app = express();
-    let s;
+    let server;
 
     app.get('/', function (req, res) {
       res.send('hello');
     });
 
-    s = app.listen(function () {
-      const url = 'http://localhost:' + s.address().port;
+    server = app.listen(function () {
+      const url = 'http://localhost:' + server.address().port;
       request(url)
         .get('/')
         .expect('hello', done);
@@ -37,14 +44,14 @@ describe('request(url)', function () {
   describe('.end(cb)', function () {
     it('should set `this` to the test object when calling cb', function (done) {
       const app = express();
-      let s;
+      let server;
 
       app.get('/', function (req, res) {
         res.send('hello');
       });
 
-      s = app.listen(function () {
-        const url = 'http://localhost:' + s.address().port;
+      server = app.listen(function () {
+        const url = 'http://localhost:' + server.address().port;
         const test = request(url).get('/');
         test.end(function (err, res) {
           this.should.eql(test);
@@ -80,7 +87,7 @@ describe('request(app)', function () {
       res.send('hey');
     });
 
-    server = app.listen(4000, function () {
+    server = app.listen(function () {
       request(server)
         .get('/')
         .end(function (err, res) {
@@ -93,13 +100,15 @@ describe('request(app)', function () {
 
   it('should work with remote server', function (done) {
     const app = express();
+    let server;
 
     app.get('/', function (req, res) {
       res.send('hey');
     });
 
-    app.listen(4001, function () {
-      request('http://localhost:4001')
+    server = app.listen(function () {
+      const url = 'http://localhost:' + server.address().port;
+      request(url)
         .get('/')
         .end(function (err, res) {
           res.status.should.equal(200);
@@ -748,9 +757,7 @@ describe('request(app)', function () {
 
       it('reports errors', function (done) {
         get
-          .expect(function (res) {
-            throw new Error('failed');
-          })
+          .expect(throwError('failed'))
           .end(function (err) {
             err.message.should.equal('failed');
             shouldIncludeStackWithThisFile(err);
@@ -758,6 +765,25 @@ describe('request(app)', function () {
           });
       });
 
+      // this scenario should never happen after https://github.com/visionmedia/supertest/pull/767
+      // meant for test coverage for lib/test.js#287
+      // https://github.com/visionmedia/supertest/blob/e064b5ae71e1dfa3e1a74745fda527ac542e1878/lib/test.js#L287
+      it('_assertFunction should catch and return error', function (done) {
+        const error = new Error('failed');
+        const returnedError = get
+          // private api
+          ._assertFunction(function (res) {
+            throw error;
+          });
+        get
+          .end(function () {
+            returnedError.should.equal(error);
+            returnedError.message.should.equal('failed');
+            shouldIncludeStackWithThisFile(returnedError);
+            done();
+          });
+      });
+
       it(
         'ensures truthy non-errors returned from asserts are not promoted to errors',
         function (done) {
@@ -774,9 +800,7 @@ describe('request(app)', function () {
 
       it('ensures truthy errors returned from asserts are throw to end', function (done) {
         get
-          .expect(function (res) {
-            return new Error('some descriptive error');
-          })
+          .expect(throwError('some descriptive error'))
           .end(function (err) {
             err.message.should.equal('some descriptive error');
             shouldIncludeStackWithThisFile(err);
@@ -1269,7 +1293,7 @@ describe('request.get(url).query(vals) works as expected', function () {
       });
   });
 
-  it('handles unknown errors', function (done) {
+  it('handles unknown errors (err without res)', function (done) {
     const app = express();
 
     nock.disableNetConnect();
@@ -1285,6 +1309,8 @@ describe('request.get(url).query(vals) works as expected', function () {
       // https://github.com/visionmedia/supertest/issues/352
       .expect(200)
       .end(function (err, res) {
+        should.exist(err);
+        should.not.exist(res);
         err.should.be.an.instanceof(Error);
         err.message.should.match(/Nock: Disallowed net connect/);
         shouldIncludeStackWithThisFile(err);
@@ -1294,6 +1320,35 @@ describe('request.get(url).query(vals) works as expected', function () {
     nock.restore();
   });
 
+  // this scenario should never happen
+  // there shouldn't be any res if there is an err
+  // meant for test coverage for lib/test.js#169
+  // https://github.com/visionmedia/supertest/blob/5543d674cf9aa4547927ba6010d31d9474950dec/lib/test.js#L169
+  it('handles unknown errors (err with res)', function (done) {
+    const app = express();
+
+    app.get('/', function (req, res) {
+      res.status(200).send('OK');
+    });
+
+    const resError = new Error();
+    resError.status = 400;
+
+    const serverRes = { status: 200 };
+
+    request(app)
+      .get('/')
+      // private api
+      .assert(resError, serverRes, function (err, res) {
+        should.exist(err);
+        should.exist(res);
+        err.should.equal(resError);
+        res.should.equal(serverRes);
+        // close the server explicitly (as we are not using expect/end/then)
+        this.end(done);
+      });
+  });
+
   it('should assert using promises', function (done) {
     const app = express();
 
@@ -1310,3 +1365,62 @@ describe('request.get(url).query(vals) works as expected', function () {
       });
   });
 });
+
+const describeHttp2 = (http2) ? describe : describe.skip;
+describeHttp2('http2', function() {
+  // eslint-disable-next-line global-require
+  const proxyquire = require('proxyquire');
+
+  const tests = [
+    {
+      title: 'request(app)',
+      api: request,
+      mockApi: proxyquire('../index.js', { http2: null })
+    },
+    {
+      title: 'request.agent(app)',
+      api: request.agent,
+      mockApi: proxyquire('../lib/agent.js', { http2: null })
+    }
+  ];
+
+  tests.forEach(({ title, api, mockApi }) => {
+    describe(title, function () {
+      const app = function(req, res) {
+        res.end('hey');
+      };
+
+      it('should fire up the app on an ephemeral port', function (done) {
+        api(app, { http2: true })
+          .get('/')
+          .end(function (err, res) {
+            res.status.should.equal(200);
+            res.text.should.equal('hey');
+            done();
+          });
+      });
+
+      it('should work with an active server', function (done) {
+        const server = http2.createServer(app);
+
+        server.listen(function () {
+          api(server)
+            .get('/')
+            .http2()
+            .end(function (err, res) {
+              res.status.should.equal(200);
+              res.text.should.equal('hey');
+              // close the external server explictly
+              server.close(done);
+            });
+        });
+      });
+
+      it('should throw error if http2 is not supported', function() {
+        (function() {
+          mockApi(app, { http2: true });
+        }).should.throw('supertest: this version of Node.js does not support http2');
+      });
+    });
+  });
+});
